@@ -8,7 +8,7 @@ package Graph::Easy::Node::Anon;
 use Graph::Easy::Node;
 
 @ISA = qw/Graph::Easy::Node/;
-$VERSION = 0.02;
+$VERSION = 0.03;
 
 use strict;
 
@@ -31,8 +31,6 @@ sub _init
 
 sub _correct_size
   {
-#  my ($self,$format) = @_;
-
   $_[0];
   }
 
@@ -76,6 +74,11 @@ sub title
   '';
   }
 
+sub text_styles_as_css
+  {
+  '';
+  }
+
 1;
 __END__
 
